import env from "dotenv";
import { sendError, sendResponse } from "../utils/messages.js";
import UsuarioService from '../services/usuarioService.js';
import { ZodError } from "zod";
import { APIErro } from "../utils/ApiError.js";
import { zodCommon } from "../utils/zodCommonError.js";

env.config(); 

class systemUsuarioController {

  static listar = async (req, res) => {
    try {
        const { page = 1, limit = 10, ...filtros } = req.query;
        const { usuarios, total } = await UsuarioService.listarUsuarios(filtros, page, limit, 'desc');
        return sendResponse(res, 200, { data: usuarios, total });
    } catch (err) {
      if (err instanceof ZodError) {
        return sendError(res, 400, await zodCommon(err));

      } else if (err instanceof APIErro) {
        const { code, errors } = err.toJson();
        return sendError(res, code, ...errors);
      }

      return sendError(res, 500, "Ocorreu um erro interno no servidor!");
    }
};

static buscarPorId = async (req, res) => {
  try {

      const usuario = await UsuarioService.buscarUsuarioPorId(req.params);
      return sendResponse(res, 200, { data: usuario });

  } catch (err) {

    if (err instanceof ZodError) {
      return sendError(res, 400, await zodCommon(err));

    } else if (err instanceof APIErro) {
      const { code, errors } = err.toJson();
      return sendError(res, code, ...errors);
    }

    return sendError(res, 500, "Ocorreu um erro interno no servidor!");
  };
};


static criarUsuario = async (req, res) => {
  try {
   
    const usuario = await UsuarioService.criarUsuario(req.body);

    return sendResponse(res, 201, {
      error: false,
      message: "Requisição bem sucedida",
      data: usuario,
    })

  } catch (err) {
    
    if (err instanceof ZodError) {
      return sendError(res, 400, await zodCommon(err));

    } else if (err instanceof APIErro) {
      const { code, errors } = err.toJson();
      return sendError(res, code, ...errors);
    }

    return sendError(res, 500, "Ocorreu um erro interno no servidor!");
  }
};

static atualizar = async (req, res) => {
  try {
    let id = req.params.id;
    let parametros = {
      id: parseInt(id),
      ...req.body,
    }

    const usuario = await UsuarioService.atualizar(parametros);

    return sendResponse(res, 200, { data: usuario });
  } catch (err) {
    if (err instanceof ZodError) {
      return sendError(res, 400, await zodCommon(err));

    } else if (err instanceof APIErro) {
      const { code, errors } = err.toJson();
      return sendError(res, code, ...errors);
    }

    return sendError(res, 500, "Ocorreu um erro interno no servidor!");
  }
};



static atualizarSenha = async (req, res) => {
  try {
    let id = req.params.id
    const {senhaNova, senhaAntiga} = req.body
    
    const usuario = await UsuarioService.atualizarSenha({id, senhaAntiga, senhaNova })

      return sendResponse(res,201, {data:usuario});

    } catch (err) {

      if (err instanceof ZodError) {
        return sendError(res, 400, await zodCommon(err));

      } else if (err instanceof APIErro) {
        const { code, errors } = err.toJson();
        return sendError(res, code, ...errors);
      }

      return sendError(res, 500, "Ocorreu um erro interno no servidor!");
    }
}


  static inserir_csv = async (req, res) => {
    try {
      if (!req.file) {
        return sendError(res, 400, ["Nenhum arquivo enviado."]);
      }

      const retorno = await UsuarioService.inserir_csv(req.file);

      return sendResponse(res, 201, { data: retorno });
    } catch (err) {
      
      if (err instanceof ZodError) {
        return sendError(res, 400, await zodCommon(err));

      } else if (err instanceof APIErro) {
        const { code, errors } = err.toJson();
        return sendError(res, code, ...errors);
      }

      return sendError(res, 500, "Ocorreu um erro interno no servidor!");
    }
  };
  
}

export default systemUsuarioController;
