const aulaRoutes = {
    "/aula": {
        get: {
            tags: ["Aulas"],
            summary: "Lista as aulas de acordo com o filtro.",
            security: [{ bearerAuth: [] }],
            parameters: [
                {
                    name: "titulo",
                    in: "query",
                    description: "Título da aula.",
                    required: false,
                    schema: {
                        type: "string"
                    }
                },
                {
                    name: "modulo_id",
                    in: "query",
                    description: "ID do módulo associado à aula.",
                    required: false,
                    schema: {
                        type: "integer"
                    }
                },
                {
                    name: "aluno_id",
                    in: "query",
                    description: "ID do aluno associado à aula.",
                    required: false,
                    schema: {
                        type: "integer"
                    }
                },
                {
                    name: "page",
                    in: "query",
                    description: "Número da página para paginação (default: 1).",
                    required: false,
                    schema: {
                        type: "integer",
                        default: 1
                    }
                },
                {
                    name: "perPage",
                    in: "query",
                    description: "Número de itens por página (default: 10).",
                    required: false,
                    schema: {
                        type: "integer",
                        default: 10
                    }
                }
            ],
            responses: {
                "200": {
                    description: "Aula(s) encontrada(s) com sucesso.",
                    content: {
                        "application/json": {
                            schema: {
                                $ref: "#/components/schemas/getAllAulaRes"
                            }
                        }
                    }
                },
                "400": {
                    description: "Requisição com sintaxe incorreta ou outros problemas.",
                    content: {
                        $ref: "#/components/schemas/erro400aula"
                    }                
                },
                "404": {
                    description: "Nenhuma aula encontrada.",
                    content: {
                        $ref: "#/components/schemas/erro404aula"
                    }                
                },
                "500": {
                    description: "Servidor encontrou um erro interno.",
                    content: {
                        $ref: "#/components/schemas/erro500"
                    }                
                },
            }
        },
        post: {
            tags: ["Aulas"],
            summary: "Cria uma aula e salva seus arquivos pdf na api.",
            security: [{ bearerAuth: [] }],
            requestBody: {
                content: {
                    "application/json": {
                        schema: {
                            $ref: "#/components/schemas/criarAula"
                        }
                    }

                },
                required: true
            },
            responses: {
                "201": {
                    description: "Aula criada com sucesso.",
                    content: {
                        "application/json": {
                            schema: {
                                $ref: "#/components/schemas/createAulaRes"
                            }
                        }
                    }
                },
                "400": {
                    description: "Houve um erro em algum parâmetro do corpo da requisição.",
                    content: {
                        $ref: "#/components/schemas/erro400aula"
                    }                
                },
                "404": {
                    description: "módulo não existe.",
                    content: {
                        $ref: "#/components/schemas/erro404aula"
                    }                
                },
                "500": {
                    description: "Servidor encontrou um erro interno.",
                    content: {
                        $ref: "#/components/schemas/erro500"
                    }                
                },
            }
        }
    },
    "/aula/{id}": {  
        get: {
            tags: ["Aulas"],
            summary: "Busca uma aula pelo ID.",
            security: [{ bearerAuth: [] }],
            parameters: [
                {
                    name: "id",
                    in: "path",
                    description: "ID da aula a ser buscada.",
                    required: true,
                    schema: {
                        type: "integer"
                    }
                }
            ],
            responses: {
                "200": {
                    description: "Aula encontrada com sucesso.",
                    content: {
                        "application/json": {
                            schema: {
                                $ref: "#/components/schemas/getAllAulaRes"
                            }
                        }
                    }
                },
                "400": {
                    description: "Requisição com sintaxe incorreta ou outros problemas.",
                    content: {
                        $ref: "#/components/schemas/erro400aula"
                    }
                },
                "404": {
                    description: "Nenhuma aula encontrada.",
                    content: {
                        $ref: "#/components/schemas/erro404aula"
                    }
                },
                "500": {
                    description: "Servidor encontrou um erro interno.",
                    content: {
                        $ref: "#/components/schemas/erro500"
                    }
                }
            }
        },
        patch: {
            tags: ["Aulas"],
            summary: "Atualiza uma aula pelo ID.",
            security: [{ bearerAuth: [] }],
            parameters: [
                {
                    name: "id",
                    in: "path",
                    description: "ID da aula a ser buscada.",
                    required: true,
                    schema: {
                        type: "integer"
                    }
                }
            ],
            requestBody: {
                content: {
                    "multipart/form-data": {
                        schema: {
                            type: "object",
                            properties: {
                                modulo_id: {
                                    type: "integer",
                                    description: "ID do módulo associado à aula."
                                },
                                titulo: {
                                    type: "string",
                                    description: "título da aula a ser criada."
                                },
                                video: {
                                    type: "string",
                                    description: "link do vídeo da aula."
                                },
                                descricao: {
                                    type: "string",
                                    description: "descrição da aula a ser criada."
                                }
                            },
                        }
                    }
                },
                required: true
            },
            responses: {
                "200": {
                    description: "Aula atualizada com sucesso.",
                    content: {
                        "application/json": {
                            schema: {
                                $ref: "#/components/schemas/updateAulaRes"
                            }
                        }
                    }
                },
                "400": {
                    description: "Requisição com sintaxe incorreta ou outros problemas.",
                    content: {
                        $ref: "#/components/schemas/erro400aula"
                    }
                },
                "404": {
                    description: "Nenhuma aula encontrada.",
                    content: {
                        $ref: "#/components/schemas/erro404aula"
                    }
                },
                "500": {
                    description: "Servidor encontrou um erro interno.",
                    content: {
                        $ref: "#/components/schemas/erro500"
                    }
                }
            }
        },
        delete: {
            tags: ["Aulas"],
            summary: "Deleta uma aula pelo ID.",
            security: [{ bearerAuth: [] }],
            parameters: [
                {
                    name: "id",
                    in: "path",
                    description: "ID da aula a ser buscada.",
                    required: true,
                    schema: {
                        type: "integer"
                    }
                }
            ],
            responses: {
                "200": {
                    description: "Aula deletada com sucesso.",
                    content: {
                        "application/json": {
                            schema: {
                                $ref: "#/components/schemas/deleteAulaRes"
                            }
                        }
                    }
                },
                "400": {
                    description: "Requisição com sintaxe incorreta ou outros problemas.",
                    content: {
                        $ref: "#/components/schemas/erro400aula"
                    }
                },
                "404": {
                    description: "Nenhuma aula encontrada.",
                    content: {
                        $ref: "#/components/schemas/erro404aula"
                    }
                },
                "500": {
                    description: "Servidor encontrou um erro interno.",
                    content: {
                        $ref: "#/components/schemas/erro500"
                    }
                }
            }
        }
    },
    "/aula/status": {
        post: {
            tags: ["Aulas"],
            summary: "Coloca uma aula com o status \"feito\" para um aluno específico.",
            security: [{ bearerAuth: [] }],
            requestBody: {
                content: {
                    "application/json": {
                        schema: {
                            type: "object",
                            properties: {
                                aluno_id: {
                                    type: "integer",
                                    description: "Id do aluno."
                                },
                                aula_id: {
                                    type: "integer",
                                    description: "id da aula."
                                },
                                feito: {
                                    type: "boolean",
                                    description: "status da aula."
                                }
                            },
                            required: ["aluno_id", "aula_id", "feito"]
                        }
                    }
                },
                required: true
            },
            responses: {
                "201": {
                    description: "Aula assistida.",
                    content: {
                        "application/json": {
                            schema: {
                                $ref: "#/components/schemas/statusAulaRes"
                            }
                        }
                    }
                },
                "400": {
                    description: "Houve um erro em algum parâmetro do corpo da requisição.",
                    content: {
                        $ref: "#/components/schemas/erro400aula"
                    }                
                },
                "404": {
                    description: "Nenhuma aula encontrada.",
                    content: {
                        $ref: "#/components/schemas/erro404aula"
                    }                
                },
                "500": {
                    description: "Servidor encontrou um erro interno.",
                    content: {
                        $ref: "#/components/schemas/erro500"
                    }                
                },
            }
        }
    }
};

export default aulaRoutes;
