import AulaRepository from "../repositories/AulaRepository.js";
import AulaSchema from "../schemas/AulaSchemas.js";
import { APIErro } from "../utils/ApiError.js";
import fs from "fs";
import path from "path";

class AulaService {

    static async listar(parametros) {
        const parametrosValidados = AulaSchema.listarSchema.parse(parametros);
        const { page, perPage } = parametrosValidados;
        const offset = (page - 1) * perPage;

        const filtroFeito = AulaRepository.createFilterFeito(parametrosValidados);
        const filtroRepository = AulaRepository.createFilterAula(parametrosValidados);

        const totalAulas = await AulaRepository.countAulas(filtroRepository);

        const aulas = await AulaRepository.findAllAulas({
            ...filtroRepository,
            skip: offset,
            take: perPage
        });

        const aulasFeitasRevisadas = await AulaRepository.findAllFeitos({
            ...filtroFeito
        });

        if (aulas.length === 0) {
            return {
                aulas: null,
                feitas: null
            }
        }

        const totalPages = Math.ceil(totalAulas.id / perPage);

        return {
            aulas,
            feitas: aulasFeitasRevisadas,
            pagination: {
                totalPages,
                currentPage: page,
                perPage
            }
        };
    }


    static async listarPorID(idDoParam) {
        const IdValidado = AulaSchema.listarPorIdSchema.parse(idDoParam);

        const filtroDoRepository = AulaRepository.createFilterAula({ id: IdValidado.id });
        const aula = await AulaRepository.filtrarPorId(filtroDoRepository);

        if (!aula) {
            throw new APIErro(404, [{ message: "Nenhuma aula encontrada.", }])
        }
        return aula;
    }

    static async atualizar(parametros) {

        const parametrosValidados = AulaSchema.UpdateSchema.parse(parametros);

        const { id, modulo_id, titulo, video, descricao } = parametrosValidados;

        const aulaExist = await AulaRepository.buscarPorId(id);

        if (!aulaExist) {
            throw new APIErro(404, [{ message: "Nenhuma aula encontrada.", }])
        }

        const filtro = {
            where: { id: id },
            data: {
                modulo_id: modulo_id,
                titulo: titulo,
                video: video,
                descricao: descricao
            },
            select: {
                id: true,
                modulo_id: true,
                titulo: true,
                video: true,
                descricao: true
            }
        };

        return await AulaRepository.update(filtro);
    }

    static async create(parametros) {
        const insert = AulaSchema.schemaInsert.parse(parametros);

        const modulo = await AulaRepository.modulo_exist(insert.modulo_id);
        if (!modulo) {
            throw new APIErro(404, [{ message: "O modulo informado não existe.", }])
        }

        const AulaCriada = await AulaRepository.create(insert);
        return AulaCriada;
    }

    static async feito_status(parametros) {

        const parametrosValidados = AulaSchema.feito_status.parse(parametros);


        const aluno = await AulaRepository.buscarAluno(parametrosValidados.aluno_id);

        if (aluno == null) {
            throw new APIErro(404, [{ message: "Usuário não encontrado.", }])
        }

        const feito = await AulaRepository.buscarFeito(parametrosValidados);

        if (feito) {
            throw new APIErro(404, [{ message: "A aula já foi assistida.", }])
        }

        const AulaCriada = await AulaRepository.feito(parametrosValidados);

        return AulaCriada;
    }

    static async deletar(id) {
        const IdValidado = AulaSchema.Delet.parse(id);

        const filtroDoRepository = AulaRepository.createFilterAula({ id: IdValidado.id });
        const aulaExists = await AulaRepository.filtrarPorId(filtroDoRepository);

        if (!aulaExists) {
            throw new APIErro(404, [{ message: "Nenhuma aula encontrada.", }])
        };
        await AulaRepository.deleteArquivos(aulaExists.id)
        await AulaRepository.deleteFeito(aulaExists.id)

        const folderPath = path.join(process.cwd(), './uploads/arquivos', String(aulaExists.id));

        fs.rm(folderPath, { recursive: true, force: true }, () => { });

        const aulaDeletada = await AulaRepository.delete(aulaExists.id)

        return aulaDeletada;
    }
}

export default AulaService;
