import request from "supertest";
import { describe, expect, it} from '@jest/globals';
import app from '../../app.js'
import faker from 'faker-br';

let token = null
let aula_id_criado = null

describe('Autenticação', () => {
    it("1-Deve chamar a rota de autenticação e pegar o token", async () => {
        const req = await request(app)
        .post('/login')
        .set("Accept", "aplication/json")
        .send({
            matricula:"12345",
            senha:"senhatest"
        })
        token = req.body.data.token
    })
});

describe.skip('GET /aula - Listar todas as aulas.', () => {
    it('1- Deve retornar todas as aulas.', async () => {

        const res = await request(app)
            .get('/aula')
            .set("Accept", "application/json")
            .set("Authorization", `Bearer ${token}`);

        expect(res.status).toBe(200);
        expect(res.body.code).toBe(200);
        expect(res.body).toHaveProperty('data');
    });
    it('2- Deve retornar as aulas filtradas utilizando o titulo ou id do modulo.', async () => {

        const res = await request(app)
            .get('/aula')
            .set("Accept", "application/json")
            .set("Authorization", `Bearer ${token}`)
            .query({ titulo: 'a' });

        expect(res.status).toBe(200);
        expect(res.body.code).toBe(200);
        expect(res.body).toHaveProperty('data');
    });
    it('3- Deve retornar as aulas filtradas utilizando o id do aluno .', async () => {

        const res = await request(app)
            .get('/aula')
            .set("Accept", "application/json")
            .set("Authorization", `Bearer ${token}`)
            .query({ aluno_id: 1 });

        expect(res.status).toBe(200);
        expect(res.body.code).toBe(200);
        expect(res.body).toHaveProperty('data');
    });
    it('5- Deve retornar erro 400 quando houver um bad request', async () => {
        const res = await request(app)
            .get('/aula?aluno_id=dfsfgfg') 
            .set("Authorization", `Bearer ${token}`)

        expect(res.body.error).toBe(true);
        expect(res.status).toBe(400);
        expect(res.body.message).toBe("Requisição com sintaxe incorreta ou outros problemas.");
    });
})

describe.skip('PATCH /aula/:id - Atualizar uma aula.', () => {
    it('1- Aula deve ser atualizada e deverá retornar uma respota positiva.', async () => {
        const updatedData = {
            titulo: faker.commerce.productName(), 
            modulo_id: 2,
            video: faker.internet.url(), 
            descricao: faker.commerce.productMaterial(), 
        }
        const res = await request(app)
            .patch('/aula/1')
            .set("Accept", "application/json")
            .set("Authorization", `Bearer ${token}`)
            .send(updatedData)

        expect(res.status).toBe(200);
        expect(res.body.code).toBe(200);
        expect(res.body.error).toBe(false);
        expect(res.body).toHaveProperty('data');
        expect(res.body.message).toBe("Requisição bem sucedida.");
    });
    it('2- Deve retornar erro 404 quando não encontrar uma aula', async () => {

        const updatedData = {
            titulo: faker.commerce.productName(), 
            modulo_id: 2,
            video: faker.internet.url(), 
            descricao: faker.commerce.productMaterial(), 
        }

        const res = await request(app)
            .patch('/aula/999')
            .set("Accept", "application/json")
            .set("Authorization", `Bearer ${token}`)
            .send(updatedData)

        expect(res.status).toBe(404);
        expect(res.body.code).toBe(404);
        expect(res.body.message).toBe("O recurso solicitado não foi encontrado no servidor.");
        expect(res.body.errors[0].message).toBe("Nenhuma aula encontrada.");
    });
    
    it('3- Deve retornar erro 400 quando houver um bad request', async () => {
        const updatedData = {
            titulo: faker.commerce.productName(), 
            modulo_id: "teste",
            video: faker.internet.url(), 
            descricao: faker.commerce.productMaterial(), 
        }
        const res = await request(app)
            .patch('/aula/1')
            .set("Accept", "application/json")
            .set("Authorization", `Bearer ${token}`)
            .send(updatedData)

        expect(res.body.error).toBe(true);
        expect(res.status).toBe(400);
        expect(res.body.message).toBe("Requisição com sintaxe incorreta ou outros problemas.");
    });
});

describe.skip('POST /aula - Cria aulas e salva arquivos pdf.', () => {

    it('1- Cria uma nova aula, e salva os arquivos pdf.', async () => {

        const updatedData = {
            titulo: faker.commerce.productName(), 
            modulo_id: 1,
            video: faker.internet.url(), 
            descricao: faker.commerce.productMaterial(), 
        }

        const res = await request(app)
            .post('/aula')
            .set("Accept", "application/json")
            .set("Authorization", `Bearer ${token}`)
            .send(updatedData)

            aula_id_criado = res.body.data.id
        
        expect(res.body.error).toEqual(false)
        expect(res.status).toBe(201)
        expect(res.body.message).toEqual("Requisição bem sucedida, recurso foi criado")
        expect(res.body.data).toBeInstanceOf(Object)
        expect(res.body.data.id).toBeDefined()

    })

    it('2- Gera erro se o tipo de um dado for errado.', async () => {

        const updatedData = {
            titulo: faker.commerce.productName(), 
            modulo_id: "teste",
            video: faker.internet.url(), 
            descricao: faker.commerce.productMaterial(), 
        }

        const res = await request(app)
        .post('/aula')
        .set("Accept", "application/json")
        .set("Authorization", `Bearer ${token}`)
        .send(updatedData)
        
        expect(res.body.error).toEqual(true)
        expect(res.status).toBe(400)
        expect(res.body.message).toEqual("Requisição com sintaxe incorreta ou outros problemas.")
    })

    it('3- Gera erro se o moduo_id não existir.', async () => {

        const updatedData = {
            titulo: faker.commerce.productName(), 
            modulo_id: 123123434,
            video: faker.internet.url(), 
            descricao: faker.commerce.productMaterial(), 
        }

        const res = await request(app)
        .post('/aula')
        .set("Accept", "application/json")
        .set("Authorization", `Bearer ${token}`)
        .send(updatedData)
        
        expect(res.body.error).toEqual(true)
        expect(res.status).toBe(404)
        expect(res.body.message).toEqual("O recurso solicitado não foi encontrado no servidor.")
    })
})

describe.skip('POST /aula/status - Coloca o status de feito para uma tarefa de um certo aluno.', () => {
    
    it('1- Deve atualizar o status do aluno, para fazer com que certa aula tenha sido feita por ele.', async () => {
        const status = {
            aluno_id: 1, 
            aula_id: aula_id_criado,
            feito: true, 
        }
        const res = await request(app)
            .post('/aula/status')
            .set("Accept", "application/json")
            .set("Authorization", `Bearer ${token}`)
            .send(status)
        
        expect(res.body.error).toEqual(false)
        expect(res.status).toBe(201)
        expect(res.body.message).toEqual("Requisição bem sucedida, recurso foi criado")
        expect(res.body.data).toBeInstanceOf(Object)
        expect(res.body.data.id).toBeDefined()

    })

    it('2- Gera erro se for uma bad request.', async () => {

        const status = {
            aluno_id: "abs", 
            aula_id: 1,
            feito: true, 
        }
        const res = await request(app)
            .post('/aula/status')
            .set("Accept", "application/json")
            .set("Authorization", `Bearer ${token}`)
            .send(status)
        
        expect(res.body.error).toBe(true);
        expect(res.status).toBe(400);
        expect(res.body.message).toBe("Requisição com sintaxe incorreta ou outros problemas.");
        expect(res.body.errors[0].message).toBe("ID informado não é do tipo number");
    })

    it('3-  Deve retornar erro 404 quando não encontrar um aluno.', async () => {

        const status = {
            aluno_id: 190909, 
            aula_id: 1,
            feito: true, 
        }
            const res = await request(app)
            .post('/aula/status')
            .set("Accept", "application/json")
            .set("Authorization", `Bearer ${token}`)
            .send(status)
        
            expect(res.status).toBe(404);
            expect(res.body.code).toBe(404);
            expect(res.body.message).toBe("O recurso solicitado não foi encontrado no servidor.");
    })
})

describe.skip('DELETE /aula/:id - Dele uma aula através do id dela.', () => {
    it('1- Deve deletar uma aula utilizando o ID.', async () => {

        const res = await request(app)
            .delete("/aula/"+aula_id_criado)
            .set("Accept", "application/json")
            .set("Authorization", `Bearer ${token}`)

        expect(res.status).toBe(200);
        expect(res.body.code).toBe(200);
        expect(res.body.error).toBe(false);
        expect(res.body).toHaveProperty('data');
        expect(res.body.message).toBe("Requisição bem sucedida.");
    });
    it('2- Deve retornar erro 404 quando não encontrar uma aula', async () => {

        const res = await request(app)
            .delete('/aula/999')
            .set("Accept", "application/json")
            .set("Authorization", `Bearer ${token}`)

        expect(res.status).toBe(404);
        expect(res.body.code).toBe(404);
        expect(res.body.message).toBe("O recurso solicitado não foi encontrado no servidor.");
        expect(res.body.errors[0].message).toBe("Nenhuma aula encontrada.");
    });
    
    it('3- Deve retornar erro 400 quando houver um bad request', async () => {
        const res = await request(app)
            .delete('/aula/ASduasidg')
            .set("Authorization", `Bearer ${token}`)

        expect(res.body.error).toBe(true);
        expect(res.status).toBe(400);
        expect(res.body.message).toBe("Requisição com sintaxe incorreta ou outros problemas.");
        expect(res.body.errors[0].message).toBe("ID informado não é do tipo number");
    });
});