import { APIErro } from '../../utils/ApiError.js';
import AulaController from '../../controllers/AulaController.js';
import { sendError, sendResponse } from '../../utils/messages.js';
import { ZodError } from 'zod';
import AulaService from '../../services/AulaService.js';
import { zodCommon } from '../../utils/zodCommonError.js';

jest.mock('../../services/AulaService.js');
jest.mock('../../utils/messages.js');
jest.mock('../../utils/zodCommonError.js');

describe('1 - (GET)AulaController listarAll', () => {
        let req, res;

        beforeEach(() => {
          jest.clearAllMocks();
          req = {
            query: {}
          };
          res = {
            status: jest.fn().mockReturnThis(),
            json: jest.fn()
          };
        });

        test('Deve definir modulo_id como undefined se não for fornecido', async () => {
          req.query = {
            titulo: 'Aula Teste',
            page: '1',
            perPage: '5'
          };

          AulaService.listar.mockResolvedValue([]);

          await AulaController.listarAll(req, res);

          expect(AulaService.listar).toHaveBeenCalledWith({
            titulo: 'Aula Teste',
            modulo_id: undefined,
            aluno_id: undefined,
            page: 1,
            perPage: 5
          });
          expect(sendResponse).toHaveBeenCalledWith(res, 200, { data: [] });
        });

        test('Deve definir modulo_id como inteiro se for fornecido', async () => {
          req.query = {
            titulo: 'Aula Teste',
            modulo_id: '2',
            page: '1',
            perPage: '5'
          };

          AulaService.listar.mockResolvedValue([]);

          await AulaController.listarAll(req, res);

          expect(AulaService.listar).toHaveBeenCalledWith({
            titulo: 'Aula Teste',
            modulo_id: 2,
            aluno_id: undefined,
            page: 1,
            perPage: 5
          });
          expect(sendResponse).toHaveBeenCalledWith(res, 200, { data: [] });
        });

        test('Deve definir aluno_id como inteiro se for fornecido', async () => {
          req.query = {
            titulo: 'Aula Teste',
            aluno_id: '3',
            page: '1',
            perPage: '5'
          };

          AulaService.listar.mockResolvedValue([]);

          await AulaController.listarAll(req, res);

          expect(AulaService.listar).toHaveBeenCalledWith({
            titulo: 'Aula Teste',
            modulo_id: undefined,
            aluno_id: 3,
            page: 1,
            perPage: 5
          });
          expect(sendResponse).toHaveBeenCalledWith(res, 200, { data: [] });
        });

        test('Deve retornar erro 400 se ocorrer um erro de validação Zod', async () => {
          const mockError = new ZodError([]);
          zodCommon.mockResolvedValue('Erro de validação');
          AulaService.listar.mockRejectedValue(mockError);

          await AulaController.listarAll(req, res);

          expect(sendError).toHaveBeenCalledWith(res, 400, 'Erro de validação');
        });

        test('Deve retornar erro 500 se ocorrer um erro desconhecido', async () => {
          const mockError = new Error('Erro desconhecido');
          AulaService.listar.mockRejectedValue(mockError);

          await AulaController.listarAll(req, res);

          expect(sendError).toHaveBeenCalledWith(res, 500, 'Ocorreu um erro interno no servidor!');
        });
      });
describe('2 - (GET)AulaController listarPorID', () => {
  let req, res;

  beforeEach(() => {
      jest.clearAllMocks();
  });

  test('Deve retornar erro 500 se ocorrer um erro desconhecido', async () => {
      const mockError = new Error('Erro desconhecido');
      AulaService.listarPorID.mockRejectedValue(mockError);

      await AulaController.listarPorID(req, res);

      expect(sendError).toHaveBeenCalledWith(res, 500, 'Ocorreu um erro interno no servidor!');
  });
});
describe('3 - (UPDATE)AulaController atualizar', () => {
  let req, res;

  beforeEach(() => {
      jest.clearAllMocks();
  });

  test('Deve retornar erro 500 se ocorrer um erro desconhecido', async () => {
      const mockError = new Error('Erro desconhecido');
      AulaService.atualizar.mockRejectedValue(mockError);

      await AulaController.atualizar(req, res);

      expect(sendError).toHaveBeenCalledWith(res, 500, 'Ocorreu um erro interno no servidor!');
  });
});
describe('4 - (POST)AulaController inserir', () => {
  let req, res;

  beforeEach(() => {
      jest.clearAllMocks();
      req = {
          body: {
              modulo_id: 1,
              titulo: 'Aula Teste',
              video: 'video.mp4',
              descricao: 'Descrição da aula'
          },
          files: {
              pdf_questoes: [{ filename: 'questoes.pdf' }],
              pdf_resolucao: [{ filename: 'resolucao.pdf' }]
          }
      };
      res = {
          status: jest.fn().mockReturnThis(),
          json: jest.fn()
      };
  });

  test('Deve retornar erro 400 se ocorrer um erro de validação Zod', async () => {
      const mockError = new ZodError([]);
      zodCommon.mockResolvedValue('Erro de validação');
      AulaService.create.mockRejectedValue(mockError);

      await AulaController.inserir(req, res);

      expect(sendError).toHaveBeenCalledWith(res, 400, 'Erro de validação');
  });

  test('Deve retornar erro 500 se ocorrer um erro desconhecido', async () => {
      const mockError = new Error('Erro desconhecido');
      AulaService.create.mockRejectedValue(mockError);

      await AulaController.inserir(req, res);

      expect(sendError).toHaveBeenCalledWith(res, 500, 'Ocorreu um erro interno no servidor!');
  });
});

describe('5 - (POST)AulaController feito_status', () => {
  let req, res;

  beforeEach(() => {
      jest.clearAllMocks();
  });

  test('Deve retornar erro 500 se ocorrer um erro desconhecido', async () => {
      const mockError = new Error('Erro desconhecido');
      AulaService.feito_status.mockRejectedValue(mockError);

      await AulaController.feito_status(req, res);

      expect(sendError).toHaveBeenCalledWith(res, 500, 'Ocorreu um erro interno no servidor!');
  });
});

describe('7 - (DELETE)AulaController deletar', () => {
  let req, res;

  beforeEach(() => {
      jest.clearAllMocks();
  });

  test('Deve retornar erro 500 se ocorrer um erro desconhecido', async () => {
      const mockError = new Error('Erro desconhecido');
      AulaService.deletar.mockRejectedValue(mockError);

      await AulaController.deletar(req, res);

      expect(sendError).toHaveBeenCalledWith(res, 500, 'Ocorreu um erro interno no servidor!');
  }); 
});

// AulaController.js       |   91.13 |       75 |     100 |   91.13 | 67,93,118,125,152,169,192