import AulaRepository from '../../repositories/AulaRepository.js';  
import { prisma } from '../../configs/prismaClient.js'; 


jest.mock('../../configs/prismaClient.js', () => ({
  prisma: {
    aula: {
      findMany: jest.fn(),
      findUnique: jest.fn(),
      update: jest.fn(),
      create: jest.fn(),
      delete: jest.fn(),
    },
    feito: {
      findMany: jest.fn(),
      findFirst: jest.fn(),
      create: jest.fn(),
      deleteMany: jest.fn(),
    },
    modulo: {
      findFirst: jest.fn(),
    },
    usuario: {
      findFirst: jest.fn(),
    },
  },
}));


describe('AulaRepository', () => {
  
  beforeEach(() => {
    jest.clearAllMocks();
  });

  test('findAllFeitos', async () => {
    const mockFeitos = [{ id: 1, aluno_id: 1, aula_id: 2 }];
    prisma.feito.findMany.mockResolvedValue(mockFeitos);

    const result = await AulaRepository.findAllFeitos({});
    expect(result).toEqual(mockFeitos);
    expect(prisma.feito.findMany).toHaveBeenCalledTimes(1);
  });

  test('findAllAulas', async () => {
    const mockAulas = [{ id: 1, titulo: 'Aula 1' }];
    prisma.aula.findMany.mockResolvedValue(mockAulas);

    const result = await AulaRepository.findAllAulas({});
    expect(result).toEqual(mockAulas);
    expect(prisma.aula.findMany).toHaveBeenCalledTimes(1);
  });

  test('filtrarPorId', async () => {
    const mockAula = { id: 1, titulo: 'Aula 1' };
    prisma.aula.findUnique.mockResolvedValue(mockAula);

    const result = await AulaRepository.filtrarPorId({ where: { id: 1 } });
    expect(result).toEqual(mockAula);
    expect(prisma.aula.findUnique).toHaveBeenCalledTimes(1);
  });

  test('buscarPorId', async () => {
    const mockAula = { id: 1, titulo: 'Aula 1' };
    prisma.aula.findUnique.mockResolvedValue(mockAula);

    const result = await AulaRepository.buscarPorId(1);
    expect(result).toEqual(mockAula);
    expect(prisma.aula.findUnique).toHaveBeenCalledWith({
      where: { id: 1 },
    });
  });

  test('update', async () => {
    const mockAula = { id: 1, titulo: 'Updated Aula' };
    prisma.aula.update.mockResolvedValue(mockAula);

    const result = await AulaRepository.update({ where: { id: 1 }, data: { titulo: 'Updated Aula' } });
    expect(result).toEqual(mockAula);
    expect(prisma.aula.update).toHaveBeenCalledTimes(1);
  });

  test('create', async () => {
    const data = { titulo: 'Aula 1' };
    const mockCreatedAula = { id: 1, titulo: 'Aula 1' };
    prisma.aula.create.mockResolvedValue(mockCreatedAula);

    const result = await AulaRepository.create(data);
    expect(result).toEqual(mockCreatedAula);
    expect(prisma.aula.create).toHaveBeenCalledTimes(1);
  });

  test('delete', async () => {
    prisma.feito.deleteMany.mockResolvedValue({});
    prisma.aula.delete.mockResolvedValue({ id: 1 });

    const result = await AulaRepository.delete(1);
    expect(result).toEqual({ id: 1 });
    expect(prisma.feito.deleteMany).toHaveBeenCalledWith({ where: { aula_id: 1 } });
    expect(prisma.aula.delete).toHaveBeenCalledWith({ where: { id: 1 } });
  });

  test('createFilterAula', () => {
    const parametros = { id: 1, modulo_id: 2, titulo: 'Aula 1' };
    const filtro = AulaRepository.createFilterAula(parametros);
    expect(filtro).toEqual({
      where: {
        id: 1,
        modulo_id: 2,
        titulo: { contains: 'Aula 1' },
      },
      select: {
        id: true,
        modulo_id: true,
        titulo: true,
        video: true,
        descricao: true,
      },
    });
  });

  test('modulo_exist', async () => {
    const mockModulo = { id: 1 };
    prisma.modulo.findFirst.mockResolvedValue(mockModulo);

    const result = await AulaRepository.modulo_exist(1);
    expect(result).toEqual(mockModulo);
    expect(prisma.modulo.findFirst).toHaveBeenCalledWith({
      where: { id: 1 },
      select: { id: true },
    });
  });

  test('buscarFeito', async () => {
    const mockFeito = { id: 1 };
    prisma.feito.findFirst.mockResolvedValue(mockFeito);

    const result = await AulaRepository.buscarFeito({ aluno_id: 1, aula_id: 1, feito: true });
    expect(result).toEqual(mockFeito);
    expect(prisma.feito.findFirst).toHaveBeenCalledWith({
      where: { aluno_id: 1, aula_id: 1, feito: true },
      select: { id: true },
    });
  });

  test('feito', async () => {
    const data = { aluno_id: 1, aula_id: 1, feito: true };
    const mockFeito = { id: 1, ...data };
    prisma.feito.create.mockResolvedValue(mockFeito);

    const result = await AulaRepository.feito(data);
    expect(result).toEqual(mockFeito);
    expect(prisma.feito.create).toHaveBeenCalledWith({ data });
  });

  test('buscarAluno', async () => {
    const mockAluno = { id: 1, nome: 'Aluno 1', matricula: '123', active: true };
    prisma.usuario.findFirst.mockResolvedValue(mockAluno);

    const result = await AulaRepository.buscarAluno(1);
    expect(result).toEqual(mockAluno);
    expect(prisma.usuario.findFirst).toHaveBeenCalledWith({
      where: { id: 1 },
      select: {
        id: true,
        nome: true,
        matricula: true,
        active: true,
      },
    });
  });
});
