import { describe, expect, jest, test } from '@jest/globals';
import AulaRepository from '../../repositories/AulaRepository.js';
import AulaService from '../../services/AulaService.js';
import AulaSchemas from '../../schemas/AulaSchemas.js';
import faker from 'faker-br';
import { APIErro } from '../../utils/ApiError.js';

jest.mock('../../repositories/AulaRepository.js', () => ({
    findAllFeitos: jest.fn(),
    findAllAulas: jest.fn(),
    filtrarPorId: jest.fn(),
    createFilterAula: jest.fn(),
    createFilterFeito: jest.fn(),
    modulo_exist: jest.fn(),
    create: jest.fn(),
    buscarPorId: jest.fn(),
    update: jest.fn(),
    buscarFeito: jest.fn(),
    feito: jest.fn(),
    delete: jest.fn(),
    buscarAluno: jest.fn(),
    deleteArquivos: jest.fn(),
    deleteFeito: jest.fn(),
}));

jest.mock('../../schemas/AulaSchemas.js', () => ({
    listarSchema: {
        parse: jest.fn(),
    },
    schemaInsert: {
        parse: jest.fn(),
    },
    listarPorIdSchema: {
        parse: jest.fn(),
    },
    UpdateSchema: {
        parse: jest.fn(),
    },
    feito_status: {
        parse: jest.fn(),
    },
    Delet: {
        parse: jest.fn(),
    },
    
}));

beforeEach(() => {
    jest.clearAllMocks();
});



describe('2 - (GET) Testando o método listarPorID. -> LINHA 100', () => {
    test('1 - Deve retornar a aula correspondente ao ID fornecido. -> LINHA 128', async () => {
        const idDoParam = 1;
        const aulaMock = { id: 1, titulo: "Aula de Teste" };
        
        AulaSchemas.listarPorIdSchema.parse.mockReturnValue({ id: idDoParam });
        AulaRepository.createFilterAula.mockReturnValue({ id: idDoParam });
        AulaRepository.filtrarPorId.mockResolvedValue(aulaMock);
        
        const data = await AulaService.listarPorID(idDoParam);

        expect(data).toEqual(aulaMock);
        expect(AulaSchemas.listarPorIdSchema.parse).toHaveBeenCalledWith(idDoParam);
    });

    test('2 - Deve lançar erro se nenhuma aula for encontrada. -> LINHA 142', async () => {
        const idDoParam = 2;

        AulaSchemas.listarPorIdSchema.parse.mockReturnValue({ id: idDoParam });
        AulaRepository.createFilterAula.mockReturnValue({ id: idDoParam });
        AulaRepository.filtrarPorId.mockResolvedValue(null);
        
        await expect(AulaService.listarPorID(idDoParam)).rejects.toBeInstanceOf(APIErro);
    });
});

describe('3 - (PATCH) Testando o método que atualiza aulas.', () => {
    
    test('1 - Deve atualizar uma aula existente e retornar os dados atualizados. -> LINHA 155', async () => {
        const parametros = {
            id: 1,
            titulo: faker.commerce.productName(), 
            modulo_id: 12,
            video: faker.internet.url(), 
            descricao: faker.commerce.productMaterial(), 
            pdf_questoes: "questoes1.pdf", 
            pdf_resolucao: "gabarito1.pdf"
        };

        const aulaExistente = {
            id: 1,
            titulo: faker.commerce.productName(), 
            modulo_id: 1,
            video: faker.internet.url(), 
            descricao: faker.commerce.productMaterial(), 
            pdf_questoes: "questoes1.pdf", 
            pdf_resolucao: "gabarito1.pdf"
        };

        const aulaAtualizada = { ...parametros };

        AulaSchemas.UpdateSchema.parse.mockReturnValue(parametros);
        AulaRepository.buscarPorId.mockResolvedValue(aulaExistente);
        AulaRepository.update.mockResolvedValue(aulaAtualizada);

        const data = await AulaService.atualizar(parametros);

        expect(data).toEqual(aulaAtualizada);
    });

    test('2 - Deve lançar um erro se a aula não existir. -> LINHA 187', async () => {
        const parametros = {
            id: 1,
            modulo_id: 1,
            titulo: faker.commerce.productName()
        };

        AulaSchemas.UpdateSchema.parse.mockReturnValue(parametros);
        AulaRepository.buscarPorId.mockResolvedValue(null);

        await expect(AulaService.atualizar(parametros)).rejects.toBeInstanceOf(APIErro);
    });
});

describe('4 - (POST) Testando a criação de uma nova aula. -> LINHA 201', () => {
    test('1 - Deve criar uma nova aula sem erros.', async () => {
        const mockDeInventarios = [
            {   
                titulo: faker.commerce.productName(), 
                video: faker.internet.url(), 
                descricao: faker.commerce.productMaterial(), 
                pdf_questoes: "questoes1.pdf", 
                pdf_resolucao: "gabarito1.pdf" 
            }
        ];
    
        const parametros = {
            titulo: faker.commerce.productName(), 
            modulo_id: 1,
            video: faker.internet.url(), 
            descricao: faker.commerce.productMaterial(), 
            pdf_questoes: "questoes1.pdf", 
            pdf_resolucao: "gabarito1.pdf"
        };
    
        AulaSchemas.schemaInsert.parse.mockReturnValue(parametros);
        AulaRepository.modulo_exist.mockResolvedValue({ id: 1 });
        AulaRepository.create.mockResolvedValue(mockDeInventarios);
    
        const data = await AulaService.create(parametros);
    
        expect(data).toEqual(mockDeInventarios);
    });
    
    test('2 - Deve retornar um erro quando o modulo não existir. -> LINHA 231', async () => {
        const parametros = {
            titulo: faker.commerce.productName(), 
            modulo_id: 1,
            video: faker.internet.url(), 
            descricao: faker.commerce.productMaterial(), 
            pdf_questoes: "questoes1.pdf", 
            pdf_resolucao: "gabarito1.pdf"
        };
    
        AulaSchemas.schemaInsert.parse.mockReturnValue(parametros);
        AulaRepository.modulo_exist.mockResolvedValue(null);
    
        await expect(AulaService.create(parametros)).rejects.toBeInstanceOf(APIErro);
        expect(AulaRepository.modulo_exist).toHaveBeenCalledWith(parametros.modulo_id);
        
    });
});

describe('5 - (POST) Testando o método que registra o status de aula assistida. -> LINHA 250', () => {
    test('1 - Deve registrar uma aula como assistida e retornar os dados criados. -> LINHA 251', async () => {
        const parametros = {
            aluno_id: 1,
            aula_id: 1,
            feito:true
        };

        const aulaAssistida = {
            id: 1,
            aluno_id: parametros.aluno_id,
            aula_id: parametros.aula_id,
            feito: parametros.feito
        };

        AulaSchemas.feito_status.parse.mockReturnValue(parametros);
        AulaRepository.buscarFeito.mockResolvedValue(null);
        AulaRepository.buscarAluno.mockResolvedValue(["valor"]);
        AulaRepository.feito.mockResolvedValue(aulaAssistida);

        const data = await AulaService.feito_status(parametros);

        expect(data).toEqual(aulaAssistida);
        expect(AulaSchemas.feito_status.parse).toHaveBeenCalledWith(parametros);
    });

    test('2 - Deve lançar um erro se a aula já foi assistida. -> LINHA 275', async () => {
        const parametros = {
            aula_id: 1,
            aluno_id: 1 
        };

        const aulaExistente = {
            aula_id: parametros.aula_id,
            aluno_id: parametros.aluno_id,
            feito: true 
        };

        AulaSchemas.feito_status.parse.mockReturnValue(parametros);
        AulaRepository.buscarAluno.mockResolvedValue(["valor"]);
        AulaRepository.buscarFeito.mockResolvedValue(aulaExistente); 

        await expect(AulaService.feito_status(parametros)).rejects.toBeInstanceOf(APIErro);
    });
});

describe('6 - (DELETE) Testando o método que deleta uma aula. -> LINHA 294', () => {

    test('2 - Deve lançar um erro se a aula não existir. -> LINHA 319', async () => {
        const idDoParam = { id: 1 };

        AulaSchemas.Delet.parse.mockReturnValue(idDoParam);
        AulaRepository.createFilterAula.mockReturnValue({ id: idDoParam.id });
        AulaRepository.filtrarPorId.mockResolvedValue(null); 

        await expect(AulaService.deletar(idDoParam)).rejects.toBeInstanceOf(APIErro);
    });

    test('2 - Deve lançar um erro se a pasta não existir. -> LINHA 319', async () => {
        const idDoParam = { id: 1 };

        AulaSchemas.Delet.parse.mockReturnValue(idDoParam);
        AulaRepository.createFilterAula.mockReturnValue({ id: idDoParam.id });
        AulaRepository.filtrarPorId.mockResolvedValue({id:12344});
        AulaRepository.delete.mockResolvedValue({aula:1}); 
        AulaRepository.deleteArquivos.mockResolvedValue({}); 
        AulaRepository.deleteFeito.mockResolvedValue({}); 

        const data = await AulaService.deletar(idDoParam)

        expect(data).toEqual({aula:1});
    });
});
