import { describe, expect, jest, test, beforeEach, afterEach } from '@jest/globals';
import getSwaggerOptions from '../../docs/config/head.js';
import authPaths from "../../docs/routes/auth";
import turmaPaths from "../../docs/routes/turma.js";
import moduloRoutes from "../../docs/routes/modulo.js";
import usuarioCsvRoutes from "../../docs/routes/usuario-csv.js";
import aulaRoutes from "../../docs/routes/aula.js";
import usuarioRoutes from "../../docs/routes/usuario.js";
import arquivosRoutes from '../../docs/routes/arquivos.js';

// Mockando rotas de forma mais realista para o teste
jest.mock('../../docs/routes/auth', () => ({
  "/login": { get: { summary: "Login Route" } }
}));
jest.mock('../../docs/routes/turma.js', () => ({
  "/turma": { get: { summary: "Turma Route" } }
}));
jest.mock('../../docs/routes/modulo.js', () => ({
  "/modulo": { get: { summary: "Modulo Route" } }
}));
jest.mock('../../docs/routes/usuario-csv.js', () => ({
  "/usuario/csv": { post: { summary: "CSV Usuario Route" } }
}));
jest.mock('../../docs/routes/aula.js', () => ({
  "/aula": { get: { summary: "Aula Route" } }
}));
jest.mock('../../docs/routes/usuario.js', () => ({
  "/usuario": { get: { summary: "Usuario Route" } }
}));

describe('Testando a função getSwaggerOptions', () => {

  beforeEach(() => {
    process.env.NODE_ENV = 'production';
  });

  afterEach(() => {
    jest.resetModules();
    delete process.env.NODE_ENV;
  });

  test('Deve retornar as opções corretas para o Swagger no ambiente de produção', () => {
    const swaggerOptions = getSwaggerOptions();

    expect(swaggerOptions.swaggerDefinition.servers).toEqual([
      { url: "http://localhost:3051" },
      { url: "http://localhost:3051" }
    ]);

    expect(swaggerOptions.swaggerDefinition.info.title).toBe('Plataforma de matemática');
    expect(swaggerOptions.swaggerDefinition.info.version).toBe('1.0-alpha');
    expect(swaggerOptions.swaggerDefinition.info.description).toBe(
      'Plataforma de matemática\n\nÉ necessário autenticar com token JWT antes de utilizar as rotas, a rota /login deverá ser feita utilizando uma matricula e senha válidas.'
    );

    expect(swaggerOptions.swaggerDefinition.tags).toEqual([
      {
        name: "Login",
        description: "Rota para autenticação"
      },
      {
        name: "Aulas",
        description: "Rota para gestão de aulas"
      },
      {
        name: "Módulos",
        description: "Rota para gestão de módulos"
      },
      {
        name: "Turmas",
        description: "Rota para gestão de turmas"
      },
      {
        name: "Usuários",
        description: "Rota para gestão de usuários"
      },
      {
        name: "Usuários/csv",
        description: "Rota para inserir usuários utilizando um arquivo no formato CSV"
      },
      {
        name: "Arquivos",
        description: "Rotas para gestão de Arquivos das aulas."
      },
    ]);

    expect(swaggerOptions.swaggerDefinition.paths).toEqual({
      ...authPaths,
      ...turmaPaths,
      ...moduloRoutes,
      ...usuarioCsvRoutes,
      ...aulaRoutes,
      ...usuarioRoutes,
      ...arquivosRoutes,
    });

    expect(swaggerOptions.swaggerDefinition.components.securitySchemes.bearerAuth).toEqual({
      type: 'http',
      scheme: 'bearer',
      bearerFormat: 'JWT',
    });
    expect(swaggerOptions.swaggerDefinition.security).toEqual([
      { bearerAuth: [] }
    ]);
  });

  test('Deve retornar as opções corretas para o Swagger no ambiente de desenvolvimento', () => {
    process.env.NODE_ENV = 'development';

    const swaggerOptions = getSwaggerOptions();

    expect(swaggerOptions.swaggerDefinition.servers).toEqual([
      { url: "http://localhost:3051" },
      { url: "http://localhost:3051" }
    ]);
  });

});

