import request from 'supertest';
import express from 'express';
import loginController from '../../controllers/loginController.js';
import loginService from '../../services/loginService.js';
import { sendError, sendResponse } from '../../utils/messages.js';
import { z, ZodError } from "zod";

jest.mock('../../services/loginService.js');
jest.mock('../../utils/messages.js');

const app = express();
app.use(express.json());
app.post('/login', loginController.logar);

describe('loginController', () => {
  describe('logar', () => {
    it('should return 200 and user data on successful login', async () => {
      const mockUser = { id: 1, name: 'John Doe' };
      loginService.login.mockResolvedValue(mockUser);
      sendResponse.mockImplementation((res, status, data) => res.status(status).json(data));

      const response = await request(app)
        .post('/login')
        .send({ username: 'john', password: 'password' });

      expect(response.status).toBe(200);
      expect(response.body).toEqual(mockUser);
    });

    it('should return 500 for internal server error', async () => {
      loginService.login.mockRejectedValue(new Error('Internal server error'));
      sendError.mockImplementation((res, status, message) => res.status(status).json({ error: message }));

      const response = await request(app)
        .post('/login')
        .send({ username: 'john', password: 'password' });

      expect(response.status).toBe(500);
      expect(response.body).toEqual({ error: 'Ocorreu um erro interno no servidor!' });
    });
  });
  it('should return 200 and call loginService.login with correct parameters', async () => {
    const mockUser = { id: 1, nome: 'Daniel Silva' };
    loginService.login.mockResolvedValue({ data: mockUser });
  
    const req = { body: { matricula: "12345", senha: "senhatest" } };
    const res = {
      status: jest.fn(() => ({
        json: jest.fn()
      }))
    };
  
    await loginController.logar(req, res);
  

    expect(loginService.login).toHaveBeenCalledWith(req.body);
  

    expect(res.status).toHaveBeenCalledWith(200);
  });
  
});