import PermissaoMiddleware from '../../middlewares/permissaoMiddleware';
import { prisma } from '../../configs/prismaClient';
import jwt from 'jsonwebtoken';
import { sendError } from '../../utils/messages';

jest.mock('jsonwebtoken');
jest.mock('../../utils/messages');
jest.mock('../../configs/prismaClient', () => ({
    prisma: {
        usuario: {
            findUnique: jest.fn()
        },
        rota: {
            findMany: jest.fn()  // Mockando corretamente o findMany
        }
    }
}));


describe('PermissaoMiddleware', () => {
    let req, res, next;

    beforeEach(() => {
        req = {
            headers: {
                authorization: 'Bearer token'
            },
            route: {
                path: '/test'
            },
            method: 'GET'
        };
        res = {
            status: jest.fn().mockReturnThis(),
            json: jest.fn()
        };
        next = jest.fn();
    });

    it('should return 401 if token is not provided', async () => {
        req.headers.authorization = null;
        await PermissaoMiddleware(req, res, next);
        expect(sendError).toHaveBeenCalledWith(res, 401, { message: 'Token não fornecido' });
    });
    // it('should return 404 if user is not found', async () => {
    //     jwt.verify.mockReturnValue({ data: { _id: 'userId' } });
    //     prisma.usuario.findUnique.mockResolvedValue(null); // Mockando a resposta de usuário não encontrado
    
    //     await PermissaoMiddleware(req, res, next);
    
    //     expect(sendError).toHaveBeenCalledWith(res, 404, { message: 'Usuário não encontrado' });
    // });
    

    // it('should return 403 if user is inactive', async () => {
    //     jwt.verify.mockReturnValue({ data: { _id: 'userId' } });

    //     // Mockando o findUnique para retornar um usuário inativo
    //     prisma.usuario.findUnique.mockResolvedValue({ active: false });

    //     await PermissaoMiddleware(req, res, next);
    //     expect(sendError).toHaveBeenCalledWith(res, 403, { message: 'Usuário Inativo' });
    // });

    it('should return 403 if user does not have permission', async () => {
        jwt.verify.mockReturnValue({ data: { _id: 'userId' } });
        prisma.usuario.findUnique.mockResolvedValue({ active: true, grupo_id: 'groupId' });
        prisma.rota.findMany.mockResolvedValue([]); // Nenhuma rota encontrada para o grupo
    
        await PermissaoMiddleware(req, res, next);
    
        expect(sendError).toHaveBeenCalledWith(res, 403, { message: 'Acesso negado' });
    });
    

    it('should call next if user has permission', async () => {
        jwt.verify.mockReturnValue({ data: { _id: 'userId' } });
        prisma.usuario.findUnique.mockResolvedValue({ active: true, grupo_id: 'groupId' });
        prisma.rota.findMany.mockResolvedValue([{ rota: 'test', get: true }]); // Simulando uma rota permitida
    
        await PermissaoMiddleware(req, res, next);
    
        expect(next).toHaveBeenCalled();
    });
    
    it('should return 500 if an error occurs', async () => {
        jwt.verify.mockImplementation(() => { throw new Error('Test error'); });
    
        await PermissaoMiddleware(req, res, next);
    
        expect(sendError).toHaveBeenCalledWith(res, 500, ["OCORREU UM ERRO INTERNO"]);
    });
});
