import ModuloRepository from '../../repositories/moduloRepository'; 
import { prisma } from '../../configs/prismaClient.js';  


jest.mock('../../configs/prismaClient', () => ({
  prisma: {
    modulo: {
      findMany: jest.fn(),
      findUnique: jest.fn(),
      create: jest.fn(),
      update: jest.fn(),
      delete: jest.fn(),
    },
    turma: {
      findFirst: jest.fn(),
    },
  },
}));

describe('ModuloRepository', () => {
  beforeEach(() => {
    jest.clearAllMocks();  
  });

  test('listar', async () => {
    const mockModulos = [{ id: 1, titulo: 'Modulo 1', descricao: 'Descricao 1' }];
    prisma.modulo.findMany.mockResolvedValue(mockModulos);

    const result = await ModuloRepository.listar({});
    expect(result).toEqual(mockModulos);
    expect(prisma.modulo.findMany).toHaveBeenCalledTimes(1);
  });

  test('listarPorId', async () => {
    const mockModulo = { id: 1, titulo: 'Modulo 1', descricao: 'Descricao 1' };
    prisma.modulo.findUnique.mockResolvedValue(mockModulo);

    const result = await ModuloRepository.listarPorId({ where: { id: 1 } });
    expect(result).toEqual(mockModulo);
    expect(prisma.modulo.findUnique).toHaveBeenCalledWith({
      where: { id: 1 },
    });
  });

  test('inserir', async () => {
    const data = { turma_id: 1, titulo: 'Modulo 1', descricao: 'Descricao 1', image: 'image1.png' };
    const mockModulo = { id: 1, turma_id: 1, titulo: 'Modulo 1', descricao: 'Descricao 1', image: 'image1.png' };
    prisma.modulo.create.mockResolvedValue(mockModulo);

    const result = await ModuloRepository.inserir(data);
    expect(result).toEqual(mockModulo);
    expect(prisma.modulo.create).toHaveBeenCalledWith({
      data,
      select: {
        id: true,
        turma_id: true,
        titulo: true,
        descricao: true,
        image: true,
      },
    });
  });

  test('atualizar', async () => {
    const mockModuloAtualizado = { id: 1, titulo: 'Modulo Atualizado' };
    const dataAtualizacao = {
      where: { id: 1 },
      data: { titulo: 'Modulo Atualizado' },
    };
    prisma.modulo.update.mockResolvedValue(mockModuloAtualizado);

    const result = await ModuloRepository.atualizar(dataAtualizacao);
    expect(result).toEqual(mockModuloAtualizado);
    expect(prisma.modulo.update).toHaveBeenCalledWith(dataAtualizacao);
  });

  test('deletar', async () => {
    const mockModuloDeletado = { id: 1, titulo: 'Modulo a ser deletado' };
    prisma.modulo.delete.mockResolvedValue(mockModuloDeletado);

    const result = await ModuloRepository.deletar(1);
    expect(result).toEqual(mockModuloDeletado);
    expect(prisma.modulo.delete).toHaveBeenCalledWith({
      where: { id: 1 },
    });
  });

  test('constructFilters', () => {
    const parametros = { id: 1, titulo: 'Modulo 1', descricao: 'Descricao 1' };
    const filtroEsperado = {
      where: {
        id: 1,
        titulo: { contains: 'Modulo 1' },
        descricao: { contains: 'Descricao 1' },
      },
      select: {
        id: true,
        turma_id: true,
        titulo: true,
        descricao: true,
        image: true,
      },
    };
    const filtro = ModuloRepository.constructFilters(parametros);
    expect(filtro).toEqual(filtroEsperado);
  });

  test('turma_exist', async () => {
    const mockTurma = { id: 1 };
    prisma.turma.findFirst.mockResolvedValue(mockTurma);

    const result = await ModuloRepository.turma_exist(1);
    expect(result).toEqual(mockTurma);
    expect(prisma.turma.findFirst).toHaveBeenCalledWith({
      where: { id: 1 },
      select: { id: true },
    });
  });
});
