import usuarioRepository from '../../repositories/usuarioRepository.js'; 
import { prisma } from '../../configs/prismaClient.js';  
import bcrypt from 'bcrypt';

jest.mock('../../configs/prismaClient', () => ({
  prisma: {
    usuario: {
      findMany: jest.fn(),
      findUnique: jest.fn(),
      findFirst: jest.fn(),
      update: jest.fn(),
      create: jest.fn(),
      createMany: jest.fn(),
    },
    grupo: {
      findUnique: jest.fn(),
      findFirst: jest.fn(),
    },
    turma: {
      findMany: jest.fn(),
    },
    aluno: {
      createMany: jest.fn(),
    }
  },
}));

jest.mock('bcrypt', () => ({
  hash: jest.fn().mockResolvedValue('hashed_password'),
}));

describe.skip('usuarioRepository', () => {
  beforeEach(() => {
    jest.clearAllMocks();  // Limpa os mocks antes de cada teste
  });

  test('listarUsuarios', async () => {
    const mockUsuarios = [
      { id: 1, nome: 'Usuario 1', matricula: '123', active: true },
    ];
    prisma.usuario.findMany.mockResolvedValue(mockUsuarios);

    const result = await usuarioRepository.listarUsuarios({});
    expect(result).toEqual(mockUsuarios);
    expect(prisma.usuario.findMany).toHaveBeenCalledWith({
      where: {},
      select: {
        id: true,
        nome: true,
        matricula: true,
        active: true,
      },
    });
  });

  test('buscarUsuarioPorId', async () => {
    const mockUsuario = { id: 1, nome: 'Usuario 1', matricula: '123', active: true };
    prisma.usuario.findUnique.mockResolvedValue(mockUsuario);

    const result = await usuarioRepository.buscarUsuarioPorId(1);
    expect(result).toEqual(mockUsuario);
    expect(prisma.usuario.findUnique).toHaveBeenCalledWith({
      where: { id: 1 },
      select: {
        id: true,
        nome: true,
        matricula: true,
        active: true,
      },
    });
  });

  test('buscarId', async () => {
    const mockUsuario = { id: 1, nome: 'Usuario 1' };
    prisma.usuario.findUnique.mockResolvedValue(mockUsuario);

    const result = await usuarioRepository.buscarId(1);
    expect(result).toEqual(mockUsuario);
    expect(prisma.usuario.findUnique).toHaveBeenCalledWith({
      where: { id: 1 },
    });
  });

  test('atualizarUsuario', async () => {
    const mockUsuarioAtualizado = { id: 1, nome: 'Usuario Atualizado', matricula: '123', active: true };
    const filtroAtualizacao = { where: { id: 1 }, data: { nome: 'Usuario Atualizado' } };
    prisma.usuario.update.mockResolvedValue(mockUsuarioAtualizado);

    const result = await usuarioRepository.atualizarUsuario(filtroAtualizacao);
    expect(result).toEqual(mockUsuarioAtualizado);
    expect(prisma.usuario.update).toHaveBeenCalledWith(filtroAtualizacao);
  });

  test('buscarGrupoPorId', async () => {
    const mockGrupo = { id: 1 };
    prisma.grupo.findUnique.mockResolvedValue(mockGrupo);

    const result = await usuarioRepository.buscarGrupoPorId(1);
    expect(result).toEqual(mockGrupo);
    expect(prisma.grupo.findUnique).toHaveBeenCalledWith({
      where: { id: 1 },
      select: { id: true },
    });
  });

  test('buscarSenha', async () => {
    const mockUsuario = { id: 1, matricula: '123', senha: 'hashed_password' };
    prisma.usuario.findUnique.mockResolvedValue(mockUsuario);

    const result = await usuarioRepository.buscarSenha(1);
    expect(result).toEqual(mockUsuario);
    expect(prisma.usuario.findUnique).toHaveBeenCalledWith({
      where: { id: 1 },
      select: { id: true, matricula: true, senha: true },
    });
  });

  test('buscarUsuarioPorMatricula', async () => {
    const mockUsuario = { id: 1, nome: 'Usuario 1', matricula: '123', senha: 'hashed_password', active: true };
    prisma.usuario.findFirst.mockResolvedValue(mockUsuario);

    const result = await usuarioRepository.buscarUsuarioPorMatricula('123');
    expect(result).toEqual(mockUsuario);
    expect(prisma.usuario.findFirst).toHaveBeenCalledWith({
      where: { matricula: '123' },
      select: {
        id: true,
        nome: true,
        senha: true,
        matricula: true,
        active: true,
      },
    });
  });

  test('criarUsuario', async () => {
    const dataUsuario = { nome: 'Usuario 1', matricula: '123', senha: 'senha123' };
    const mockUsuario = { id: 1, nome: 'Usuario 1', matricula: '123', active: true };

    prisma.usuario.create.mockResolvedValue(mockUsuario);
    bcrypt.hash.mockResolvedValue('hashed_password');

    const result = await usuarioRepository.criarUsuario(dataUsuario);
    expect(result).toEqual(mockUsuario);
    expect(prisma.usuario.create).toHaveBeenCalledWith({
      data: {
        nome: 'Usuario 1',
        matricula: '123',
        senha: 'hashed_password',
      },
    });
    expect(bcrypt.hash).toHaveBeenCalledWith('senha123', parseInt(process.env.SALT, 10));
  });

  test('listar_csv', async () => {
    const mockUsuarios = [
      { nome: 'Usuario 1', matricula: '123' },
    ];
    prisma.usuario.findMany.mockResolvedValue(mockUsuarios);

    const result = await usuarioRepository.listar_csv();
    expect(result).toEqual(mockUsuarios);
    expect(prisma.usuario.findMany).toHaveBeenCalledWith({
      select: {
        nome: true,
        matricula: true,
      },
    });
  });

  test('buscar_turmas', async () => {
    const mockTurmas = [{ id: 1, nome: 'Turma 1' }];
    prisma.turma.findMany.mockResolvedValue(mockTurmas);

    const result = await usuarioRepository.buscar_turmas();
    expect(result).toEqual(mockTurmas);
    expect(prisma.turma.findMany).toHaveBeenCalledTimes(1);
  });

  test('inserir_alunos', async () => {
    const mockAlunos = [{ matricula: '123', nome: 'Aluno 1' }];
    prisma.aluno.createMany.mockResolvedValue(mockAlunos);

    const result = await usuarioRepository.inserir_alunos(mockAlunos);
    expect(result).toEqual(mockAlunos);
    expect(prisma.aluno.createMany).toHaveBeenCalledWith({
      data: mockAlunos,
    });
  });

  test('grupo_alunos', async () => {
    const mockGrupo = { id: 1, nome: 'alunos' };
    prisma.grupo.findFirst.mockResolvedValue(mockGrupo);

    const result = await usuarioRepository.grupo_alunos();
    expect(result).toEqual(mockGrupo);
    expect(prisma.grupo.findFirst).toHaveBeenCalledWith({
      where: { nome: { contains: 'alunos' } },
    });
  });

  test('inserir_usuarios', async () => {
    const mockUsuario = { id: 1, nome: 'Usuario 1', matricula: '123', grupo_id: 1, active: true };
    const insertData = { nome: 'Usuario 1', matricula: '123', grupo_id: 1, active: true };
    
    prisma.usuario.create.mockResolvedValue(mockUsuario);

    const result = await usuarioRepository.inserir_usuarios(insertData);
    expect(result).toEqual(mockUsuario);
    expect(prisma.usuario.create).toHaveBeenCalledWith({
      data: insertData,
      select: {
        id: true,
        nome: true,
        matricula: true,
        grupo_id: true,
        active: true,
      },
    });
  });
});
