import { messages, sendError, sendResponse } from './messages';
import request from 'supertest';
import express from 'express';

const app = express();
app.use(express.json());

app.get('/test-error', (req, res) => {
    sendError(res, 400, "Mensagem de erro");
});

app.get('/test-response', (req, res) => {
    sendResponse(res, 200, { data: { message: "Success" } });
});

describe('messages', () => {
    test('should return correct http code messages', () => {
        expect(messages.httpCodes[200]).toBe("Requisição bem sucedida.");
        expect(messages.httpCodes[404]).toBe("O recurso solicitado não foi encontrado no servidor.");
        expect(messages.httpCodes[500]).toBe("Servidor encontrou um erro interno.");
    });

    test('should return correct resource found message', () => {
        expect(messages.error.resourceFound("ID")).toBe("O campo ID já existe.");
    });

    test('should return correct page is not available message', () => {
        expect(messages.error.pageIsNotAvailable("Home")).toBe("A página Home não está disponível.");
    });

    test('should return correct page not contains data message', () => {
        expect(messages.error.pageNotContainsData("Home")).toBe("A página Home não contém dados.");
    });

    test('should return correct validation reference message', () => {
        const result = messages.validationReference.resourceWithReference("Recurso", "Referência");
        expect(result).toEqual({ message: "Recurso com referência em Referência. Exclusão impedida." });
    });
    test('should return correct custom validation messages for invalid fields', () => {
        expect(messages.customValidation.invalidTitulo.message).toBe("Título inválido. Verifique o formato e tente novamente.");
        expect(messages.customValidation.invalidNome.message).toBe("Nome inválido. Verifique o formato e tente novamente.");
        expect(messages.customValidation.invalidMatricula.message).toBe("Matrícula inválido. Verifique o formato e tente novamente.");
    });

    test('should return correct custom validation messages for length', () => {
        expect(messages.customValidation.lengthTooBig("Nome", 10)).toBe("O campo Nome deve ter no máximo 10 de tamanho.");
        expect(messages.customValidation.lengthTooShort("Nome", 3)).toBe("O campo Nome deve ter no mínimo 3 de tamanho.");
    });

    test('should return correct custom validation messages for value', () => {
        expect(messages.customValidation.valueTooBig("Idade", 18)).toBe("O campo Idade deve ser no máximo 18.");
        expect(messages.customValidation.valueTooSmall("Idade", 5)).toBe("O campo Idade deve ser no mínimo 5.");
    });

    test('should return correct success messages', () => {
        expect(messages.success.success).toBe("Operação concluída com sucesso.");
    });

    test('should return correct error messages', () => {
        expect(messages.error.error).toBe("Ocorreu um erro ao processar a solicitação.");
        expect(messages.error.resourceNotFound("ID")).toBe("O campo ID não foi encontrado.");
    });

    test('should return correct custom validation messages', () => {
        expect(messages.customValidation.invalidTitulo.message).toBe("Título inválido. Verifique o formato e tente novamente.");
        expect(messages.customValidation.lengthTooBig("Nome", 10)).toBe("O campo Nome deve ter no máximo 10 de tamanho.");
    });

    test('should return correct auth messages', () => {
        expect(messages.auth.authenticationFailed).toBe("Falha na autenticação. Credenciais inválidas.");
        expect(messages.auth.userNotFound("123")).toBe("Usuário com ID 123 não encontrado.");
    });
});
describe('info messages', () => {
    test('should return correct welcome message', () => {
        expect(messages.info.welcome).toBe("Bem-vindo à nossa aplicação!");
    });

    test('should return correct userLoggedIn message', () => {
        expect(messages.info.userLoggedIn("Daniel")).toBe("Usuário Daniel logado com sucesso.");
    });
});
describe('auth messages', () => {
    test('should return correct authenticationFailed message', () => {
        expect(messages.auth.authenticationFailed).toBe("Falha na autenticação. Credenciais inválidas.");
    });

    test('should return correct userNotFound message', () => {
        expect(messages.auth.userNotFound("123")).toBe("Usuário com ID 123 não encontrado.");
    });

    test('should return correct invalidPermission message', () => {
        expect(messages.auth.invalidPermission).toBe("Permissão insuficiente para executar a operação.");
    });

    test('should return correct duplicateEntry message', () => {
        expect(messages.auth.duplicateEntry("email")).toBe("Já existe um registro com o mesmo email.");
    });

    test('should return correct accountLocked message', () => {
        expect(messages.auth.accountLocked).toBe("Conta bloqueada. Entre em contato com o suporte.");
    });

    test('should return correct invalidToken message', () => {
        expect(messages.auth.invalidToken).toBe("Token inválido. Faça login novamente.");
    });

    test('should return correct timeoutError message', () => {
        expect(messages.auth.timeoutError).toBe("Tempo de espera excedido. Tente novamente mais tarde.");
    });

    test('should return correct databaseConnectionError message', () => {
        expect(messages.auth.databaseConnectionError).toBe("Erro de conexão com o banco de dados. Tente novamente mais tarde.");
    });

    test('should return correct emailAlreadyExists message', () => {
        expect(messages.auth.emailAlreadyExists("test@example.com")).toBe("O endereço de e-mail test@example.com já está em uso.");
    });

    test('should return correct invalidCredentials message', () => {
        expect(messages.auth.invalidCredentials).toBe("Credenciais inválidas. Verifique seu usuário e senha.");
    });
});
describe('sendError', () => {
    test('should send error response with correct message', async () => {
        const response = await request(app).get('/test-error');
        expect(response.status).toBe(400);
        expect(response.body).toEqual({
            data: [],
            error: true,
            code: 400,
            message: "Requisição com sintaxe incorreta ou outros problemas.",
            errors: [{ message: "Mensagem de erro" }]
        });
    });
});

describe('sendResponse', () => {
    test('should send response with correct message', async () => {
        const response = await request(app).get('/test-response');
        expect(response.status).toBe(200);
        expect(response.body).toEqual({
            error: false,
            code: 200,
            message: "Requisição bem sucedida.",
            errors: [],
            data: { message: "Success" }
        });
    });
});
